% This function converts a MATLAB figure to a (custom) styled eps file
% and a set of commands for the psfrag utility fragmaster. This is
% extremely useful when creating figures for publications.
% 
% Gustav Eje Henter 2013-06-20

function fig2fm(figname,fmclass,fmclassopt,...
    cmwidth,aspect,fontpts,axiswidth,plotwidth,marksize,scaleall,tickon)

if (nargin < 2) || isempty(fmclass),
    fmclass = 'article';
    %fmclass = 'IEEEtran';
end
if (nargin < 3) || isempty(fmclassopt),
    fmclassopt = '';
    %fmclassopt = ['a4paper,swedish,english,10pt'...
    %    ',journal,compsoc,peerreview,twoside'];
    %fmclassopt = 'a4paper,english,10pt,journal,compsoc,twoside';
end
if (nargin < 4) || isempty(cmwidth), cmwidth = 9; end
if (nargin < 5) || isempty(aspect), aspect = 4/3; end
if (nargin < 6) || isempty(fontpts), fontpts = 10; end
if (nargin < 7) || isempty(axiswidth), axiswidth = 0.45; end
if (nargin < 8) || isempty(plotwidth), plotwidth = 0.7; end
if (nargin < 9) || isempty(marksize), marksize = 4; end
if (nargin < 10) || isempty(scaleall), scaleall = 1; end
if (nargin < 11) || isempty(tickon), tickon = true; end

cmwidth = scaleall*cmwidth;
fontpts = scaleall*fontpts;
axiswidth = scaleall*axiswidth;
plotwidth = scaleall*plotwidth;
marksize = scaleall*marksize;

header = {'% fragmaster file by fig2fm.m version 2013-06-20',...
    '% ',...
    '% Page and document setup:',...
    ['% fmopt: width=' num2str(cmwidth/scaleall) 'cm'],...
    ['% fmclass: ' fmclass],...
    ['% fmclassopt: ' fmclassopt],...
    '% ',...
    '% LaTeX preamble:',...
    '% head:',...
    '% \usepackage{amsmath}',...
    '% \usepackage{amssymb}',...
    '% \usepackage{fix-cm}',...
    '% end head',...
    '',...
    '% psfrag commands:'};

open([figname '.fig']);

fid = fopen([figname '_fm'],'w+');
if (fid <= -1),
    error(['Error opening file "' figname '_fm" for writing!']);
end

fprintf(fid,'%s\n',header{:});

%open([figname '.fig']); % Moved up in order to fail early

set(gcf,'PaperUnits','centimeters');
set(gcf,'PaperSize',[cmwidth cmwidth/aspect]);
set(gcf,'PaperPositionMode','manual');
set(gcf,'PaperPosition',[0 0 get(gcf,'PaperSize')]);
set(gcf,'Units','centimeters');

keepaxis = axis;
set(gcf,'Position',[1 1 cmwidth cmwidth/aspect]);
axis(keepaxis); % Prevent axis limits from changing

set(gca,'LineWidth',axiswidth);
set(gca,'FontName','cmr10');
%set(gca,'FontName','CMU Serif Roman');
set(gca,'FontSize',fontpts);
%set(gca,'FontSize',fontscale*get(gca,'FontSize'));
%set(gca,'Color','none');
%set(gcf,'Color','none');
set(gca,'Color','white');
set(gcf,'Color','white');
set(gca,'TickDir','out');
if tickon,
    set(gca,'TickLength',[0.015 0.025]); % Standard tick marks
else
    set(gca,'TickLength',[0 0]); % Disable tick marks
end

% Crop away unused space
%http://www.ligo.caltech.edu/~tfricke/matlab-tricks/plot-positions/html/
%tight_plot.html
%set(gca,'Position',get(gca,'OuterPosition') - ...
%    get(gca,'TightInset')*[-1 0 1 0; 0 -1 0 1; 0 0 1 0; 0 0 0 1]);

set(gca,'XTickMode','manual');
set(gca,'YTickMode','manual');

grid off;
box off;
legend boxoff;

childhs = get(gca,'Children');
childhs = reshape(childhs,1,numel(childhs));
for ch = childhs,
    if isprop(ch,'LineWidth'),
        set(ch,'LineWidth',plotwidth);
    end
    if isprop(ch,'MarkerSize'),
        set(ch,'MarkerSize',marksize);
    end
end

subst_fm_text(fid,'Title','title','Bc',1,'','');
subst_fm_text(fid,'XLabel','xlabel','Bc',1,'','');
subst_fm_text(fid,'YLabel','ylabel','Bc',1,'','');

if strcmp(get(gca,'XScale'),'log'),
    subst_fm_text(fid,'XTickLabel','xt','Bc',1,'$10^{','}$');
else
    subst_fm_text(fid,'XTickLabel','xt','Bc',1,'$','$');
end

if strcmp(get(gca,'YScale'),'log'),
    subst_fm_text(fid,'YTickLabel','yt','Br',1,'$10^{','}$');
else
    subst_fm_text(fid,'YTickLabel','yt','Br',1,'$','$');
end

legendstrs = get(legend,'String');
for li = 1:numel(legendstrs),
    lstr = legendstrs{li}; 
    if ~isempty(lstr),
        lstr = deblank(lstr);
        lstr = regexprep(lstr,'[\$_^\\{}]','');
        lstr = regexprep(lstr,'[^0-9A-Za-z]','n');
        %if (numel(lstr) > 1),
        %    lstr = lstr(1:end-1);
        %end
        %fprintf(fid,'%s\n',['\psfrag{' lstr int2str(li) '}'...
        %    '[Bc][Bc][1]{' legendstrs{li} '}']);
        fprintf(fid,'%s\n',['\psfrag{' lstr int2str(li) '}'...
            '[Bl][Bl][1]{' legendstrs{li} '}']);
        legendstrs{li} = [lstr int2str(li)];
    end
end
set(legend,'String',legendstrs);

fclose(fid);

saveas(gcf,[figname '_fm'],'fig');
saveas(gcf,[figname '_fm.eps'],'epsc');

close(gcf);

function subst_fm_text(fid,property,tag,align,scale,ltx,rtx)

str = get(gca,property);

if (size(str,1) > 1),
    ostr = {};
    for l = 1:size(str,1),
        if isempty(deblank(str(l,:))),
            continue;
        end
        fprintf(fid,'%s\n',['\psfrag{' tag int2str(l) '}['...
            align '][' align '][' num2str(scale) ']{'...
            ltx deblank(str(l,:)) rtx '}']);
        ostr{l} = [tag int2str(l)];
    end
    set(gca,property,ostr);
else
    texthandle = get(gca,property);
    str = get(texthandle,'String');
    if isempty(str),
        return;
    end
    fprintf(fid,'%s\n',['\psfrag{' tag '}['...
        align '][' align '][' num2str(scale) ']{' ltx str rtx '}']);
    set(texthandle,'String',tag);
end